@@ -2,6 +2,7 @@
  * PROJECT: GEM-Tools library
  * FILE: gt_input_file.h
  * DATE: 01/06/2012
+ * AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
  * DESCRIPTION: // TODO
  */
 
@@ -9,6 +10,10 @@
 #define GT_INPUT_FILE_H_
 
 #include "gt_commons.h"
+#include "gt_data_attributes.h"
+
+#include <zlib.h>
+#include <bzlib.h>
 
 // Codes gt_status
 #define GT_INPUT_FILE_OK 0
@@ -23,37 +28,16 @@
   GT_NULL_CHECK(input_file); \
   GT_NULL_CHECK(input_file->file_name)
 
-/*
- * File specifics (formats, attributes, ...)
- */
-typedef enum { MULTI_FASTA, FASTQ, MAP, SAM, FILE_FORMAT_UNKNOWN } gt_file_format;
-// MAP specific info
-typedef struct {
-  bool contains_qualities;
-} gt_map_file_format;
-// FASTQ/FASTA specific info
-typedef struct {
-  bool contains_qualities;
-} gt_fasta_file_format;
-// SAM specific info (headers)
-typedef struct {
-  // gt_reference_sequences reference_sequences;
-  char* program_name;
-  char* program_version;
-  /* ... */
-} gt_sam_headers;
-/* */
-
-
 /*
  * GT Input file
  */
-typedef enum { STREAM, REGULAR_FILE, MAPPED_FILE } gt_file_type;
+typedef enum { FASTA, MAP, SAM, FILE_FORMAT_UNKNOWN } gt_file_format;
+typedef enum { STREAM, REGULAR_FILE, MAPPED_FILE, GZIPPED_FILE, BZIPPED_FILE } gt_file_type;
 typedef struct {
   /* Input file */
   char* file_name;
   gt_file_type file_type;
-  FILE* file;
+  void* file;
   int fildes;
   bool eof;
   uint64_t file_size;
@@ -61,7 +45,7 @@ typedef struct {
   gt_file_format file_format;
   union {
     gt_map_file_format map_type;
-    gt_fasta_file_format fastq_type;
+    gt_fasta_file_format fasta_type;
     gt_sam_headers sam_headers;
   };
   pthread_mutex_t input_mutex;
@@ -85,14 +69,16 @@ gt_status gt_input_file_close(gt_input_file* const input_file);
 
 /*
  * Advanced I/O
- *   // TODO: Nested conditions would need ...
+ *   // TODO
  */
-gt_input_file* gt_input_file_segmented_file_open(
-    char* const file_name,const bool mmap_file,
-    const uint64_t segment_number,const uint64_t total_segments);
-gt_input_file* gt_input_file_reads_segmented_file_open(
-    char* const file_name,const bool mmap_file,
-    const uint64_t num_init_line,const uint64_t num_end_line);
+
+//gt_input_file* gt_input_file_segmented_file_open(
+//    char* const file_name,const bool mmap_file,
+//    const uint64_t segment_number,const uint64_t total_segments);
+//gt_input_file* gt_input_file_reads_segmented_file_open(
+//    char* const file_name,const bool mmap_file,
+//    const uint64_t num_init_line,const uint64_t num_end_line);
+
 /* Format detection */
 gt_file_format gt_input_file_detect_file_format(gt_input_file* const input_file);
 
@@ -107,11 +93,13 @@ GT_INLINE uint64_t gt_input_file_next_id(gt_input_file* const input_file);
  * Basic line functions
  */
 GT_INLINE size_t gt_input_file_dump_to_buffer(gt_input_file* const input_file,gt_vector* const buffer_dst);
+GT_INLINE size_t gt_input_file_fill_buffer(gt_input_file* const input_file);
 GT_INLINE size_t gt_input_file_next_line(gt_input_file* const input_file,gt_vector* const buffer_dst);
-GT_INLINE size_t gt_input_file_next_map_record(
-    gt_input_file* const input_file,gt_vector* const buffer_dst,uint64_t* const num_blocks);
-GT_INLINE size_t gt_input_file_next_sam_record(
-    gt_input_file* const input_file,gt_vector* const buffer_dst,gt_string* const first_field);
+
+GT_INLINE size_t gt_input_file_next_record(
+    gt_input_file* const input_file,gt_vector* const buffer_dst,gt_string* const first_field,
+    uint64_t* const num_spaces,uint64_t* const num_tabs);
+GT_INLINE bool gt_input_file_next_record_cmp_first_field(gt_input_file* const input_file,gt_string* const first_field);
 
 /*
  * Line Readers (thread-unsafe, must call mutex functions before)
@@ -124,14 +112,44 @@ GT_INLINE uint64_t gt_input_file_get_lines(
 /*
  * Processing Macros (direct parsing from input file)
  */
-#define GT_INPUT_FILE_CHECK__FILL_BUFFER(input_file,buffer_dst) \
+#define GT_INPUT_FILE_CHECK_BUFFER(input_file) \
+  if (gt_expect_false(input_file->buffer_pos >= input_file->buffer_size)) { \
+    gt_input_file_fill_buffer(input_file); \
+  }
+#define GT_INPUT_FILE_CHECK_BUFFER__DUMP(input_file,buffer_dst) \
   if (gt_expect_false(input_file->buffer_pos >= input_file->buffer_size)) { \
     if (gt_expect_true(buffer_dst!=NULL)) gt_input_file_dump_to_buffer(input_file,buffer_dst); \
     gt_input_file_fill_buffer(input_file); \
   }
-#define GT_INPUT_FILE_NEXT_CHAR(input_file,buffer_dst) \
+
+#define GT_INPUT_FILE_NEXT_CHAR(input_file) \
   ++input_file->buffer_pos; \
-  GT_INPUT_FILE_CHECK__FILL_BUFFER(input_file,buffer_dst)
+  GT_INPUT_FILE_CHECK_BUFFER(input_file)
+#define GT_INPUT_FILE_NEXT_CHAR__DUMP(input_file,buffer_dst) \
+  ++input_file->buffer_pos; \
+  GT_INPUT_FILE_CHECK_BUFFER__DUMP(input_file,buffer_dst)
+
+#define GT_INPUT_FILE_SKIP_EOL(input_file) \
+  if (!input_file->eof) { \
+    GT_INPUT_FILE_NEXT_CHAR(input_file); /* Skip EOF/DOS_EOL */  \
+    if (gt_expect_false(!input_file->eof && GT_INPUT_FILE_CURRENT_CHAR(input_file)==EOL)) { \
+      GT_INPUT_FILE_NEXT_CHAR(input_file); /* Skip DOS_EOL */  \
+    } \
+  }
+#define GT_INPUT_FILE_HANDLE_EOL(input_file,buffer_dst) \
+  if (!input_file->eof) { \
+    GT_INPUT_FILE_NEXT_CHAR__DUMP(input_file,buffer_dst); /* Skip EOF/DOS_EOL */  \
+    if (gt_expect_false(!input_file->eof && GT_INPUT_FILE_CURRENT_CHAR(input_file)==EOL)) { \
+      ++input_file->buffer_pos; \
+      if (gt_expect_true(buffer_dst!=NULL)) { \
+        gt_input_file_dump_to_buffer(input_file,buffer_dst); \
+        gt_vector_dec_used(buffer_dst); \
+        *gt_vector_get_last_elm(buffer_dst,char)=EOL; \
+      } \
+      gt_input_file_fill_buffer(input_file); \
+    } \
+  }
+
 #define GT_INPUT_FILE_CURRENT_CHAR(input_file) input_file->file_buffer[input_file->buffer_pos]
 
 
