@@ -2,6 +2,7 @@
  * PROJECT: GEM-Tools library
  * FILE: gt_input_map_parser.h
  * DATE: 01/06/2012
+ * AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
  * DESCRIPTION: // TODO
  */
 
@@ -10,10 +11,12 @@
 
 #include "gt_commons.h"
 #include "gt_dna_string.h"
+#include "gt_dna_read.h"
 
 #include "gt_input_file.h"
 #include "gt_buffered_input_file.h"
 #include "gt_input_parser.h"
+#include "gt_input_fasta_parser.h"
 
 #include "gt_buffered_output_file.h"
 
@@ -21,9 +24,9 @@
 #include "gt_template_utils.h"
 
 // Codes gt_status
-#define GT_IMP_OK 1
-#define GT_IMP_FAIL -1
-#define GT_IMP_EOF 0
+#define GT_IMP_OK   GT_STATUS_OK
+#define GT_IMP_FAIL GT_STATUS_FAIL
+#define GT_IMP_EOF  0
 
 /*
  * Parsing error/state codes // TODO Move common errors to input_parser.h
@@ -82,12 +85,13 @@
 #define GT_MAP_SCORE_SEP '/'
 
 #define GT_MAP_SEP_S ":"
+#define GT_MAP_NONE_S "-"
 #define GT_MAP_NEXT_S ","
 #define GT_MAP_MCS_S "+"
-#define GT_MAP_TEMPLATE_SEP "::"
-#define GT_MAP_TEMPLATE_SCORE ":::"
 #define GT_MAP_COUNTS_TIMES_S "x"
 #define GT_MAP_COUNTS_NOT_UNIQUE_S "!"
+#define GT_MAP_TEMPLATE_SEP "::"
+#define GT_MAP_TEMPLATE_SCORE ":::"
 
 #define GT_MAP_STRAND_FORWARD_SYMBOL '+'
 #define GT_MAP_STRAND_FORWARD_LETTER 'F'
@@ -110,6 +114,38 @@
  */
 typedef enum {PARSE_READ, PARSE_READ__MAPS, PARSE_ALL} gt_lazy_parse_mode;
 
+/*
+ * Parsing attributes
+ */
+typedef struct {
+  bool force_read_paired; // Forces to read paired reads
+  uint64_t max_parsed_maps; // Maximum number of maps to be parsed
+  gt_string* src_text; // Src text line parsed
+  bool skip_based_model; // Allows only mismatches & skips in the cigar string
+  bool remove_duplicates; // TODO
+  gt_lazy_parse_mode parse_mode;
+} gt_map_parser_attr;
+
+#define GT_MAP_PARSER_ATTR_DEFAULT(_force_read_paired) { \
+  .force_read_paired=_force_read_paired,  \
+  .max_parsed_maps=GT_ALL,  \
+  .src_text=NULL, \
+  .skip_based_model=false, \
+  .remove_duplicates=false, \
+  .parse_mode=PARSE_ALL \
+}
+
+GT_INLINE gt_map_parser_attr* gt_input_map_parser_attributes_new(const bool force_read_paired);
+GT_INLINE void gt_input_map_parser_attributes_delete(gt_map_parser_attr* const attributes);
+
+GT_INLINE void gt_input_map_parser_attributes_reset_defaults(gt_map_parser_attr* const attributes);
+GT_INLINE bool gt_input_map_parser_attributes_is_paired(gt_map_parser_attr* const attributes);
+GT_INLINE void gt_input_map_parser_attributes_set_paired(gt_map_parser_attr* const attributes,const bool force_read_paired);
+GT_INLINE void gt_input_map_parser_attributes_set_max_parsed_maps(gt_map_parser_attr* const attributes,const uint64_t max_parsed_maps);
+GT_INLINE void gt_input_map_parser_attributes_set_src_text(gt_map_parser_attr* const attributes,gt_string* const src_text);
+GT_INLINE void gt_input_map_parser_attributes_set_skip_model(gt_map_parser_attr* const attributes,const bool skip_based_model);
+GT_INLINE void gt_input_map_parser_attributes_set_duplicates_removal(gt_map_parser_attr* const attributes,const bool remove_duplicates);
+
 /*
  * MAP File basics
  */
@@ -120,16 +156,20 @@ GT_INLINE void gt_input_map_parser_prompt_error(
     uint64_t line_num,uint64_t column_pos,const gt_status error_code);
 GT_INLINE void gt_input_map_parser_next_record(gt_buffered_input_file* const buffered_map_input);
 GT_INLINE gt_status gt_input_map_parser_reload_buffer(
-    gt_buffered_input_file* const buffered_map_input,const bool synchronized_map);
+    gt_buffered_input_file* const buffered_map_input,const bool synchronized_map,const uint64_t num_lines);
 
 /*
  * MAP string parsers
  */
 GT_INLINE gt_status gt_input_map_parse_template(char* const string,gt_template* const template);
 GT_INLINE gt_status gt_input_map_parse_alignment(char* const string,gt_alignment* const alignment);
+
 GT_INLINE gt_status gt_input_map_parse_counters(char* const string,gt_vector* const counters,gt_shash* const attributes);
+
 GT_INLINE gt_status gt_input_map_parse_map(char* const string,gt_map* const map);
-GT_INLINE gt_status gt_input_map_parse_map_list(char* const string,gt_vector* const maps,const uint64_t num_maps);
+GT_INLINE gt_status gt_input_map_parse_map_list(char* const string,gt_vector* const maps);
+GT_INLINE gt_status gt_input_map_parse_map_g(char* const string,gt_map* const map,gt_map_parser_attr* const map_parser_attr);
+GT_INLINE gt_status gt_input_map_parse_map_list_g(char* const string,gt_vector* const maps,gt_map_parser_attr* const map_parser_attr);
 
 /*
  * MAP High-level Parsers
@@ -143,9 +183,28 @@ GT_INLINE gt_status gt_input_map_parser_get_template(
 GT_INLINE gt_status gt_input_map_parser_get_alignment(
     gt_buffered_input_file* const buffered_map_input,gt_alignment* const alignment);
 
-GT_INLINE gt_status gt_input_map_parser_get_template__src_text(
-    gt_buffered_input_file* const buffered_map_input,gt_template* const template,gt_string* const src_text);
-GT_INLINE gt_status gt_input_map_parser_get_alignment__src_text(
-    gt_buffered_input_file* const buffered_map_input,gt_alignment* const alignment,gt_string* const src_text);
+GT_INLINE gt_status gt_input_map_parser_get_template_g(
+    gt_buffered_input_file* const buffered_map_input,gt_template* const template,gt_map_parser_attr* const map_parser_attr);
+GT_INLINE gt_status gt_input_map_parser_get_alignment_g(
+    gt_buffered_input_file* const buffered_map_input,gt_alignment* const alignment,gt_map_parser_attr* const map_parser_attr);
+
+/*
+ * Synch read of blocks
+ */
+GT_INLINE gt_status gt_input_map_parser_synch_blocks(
+    gt_buffered_input_file* const buffered_input1,gt_buffered_input_file* const buffered_input2,pthread_mutex_t* const input_mutex);
+GT_INLINE gt_status gt_input_map_parser_synch_blocks_v(
+    pthread_mutex_t* const input_mutex,gt_map_parser_attr* const map_parser_attr,
+    uint64_t num_inputs,gt_buffered_input_file* const buffered_input,va_list v_args);
+GT_INLINE gt_status gt_input_map_parser_synch_blocks_va(
+    pthread_mutex_t* const input_mutex,gt_map_parser_attr* const map_parser_attr,
+    const uint64_t num_inputs,gt_buffered_input_file* const buffered_input,...);
+GT_INLINE gt_status gt_input_map_parser_synch_blocks_a(
+    pthread_mutex_t* const input_mutex,gt_buffered_input_file** const buffered_input,
+    const uint64_t num_inputs,gt_map_parser_attr* const map_parser_attr);
+
+GT_INLINE gt_status gt_input_map_parser_synch_blocks_by_subset(
+    pthread_mutex_t* const input_mutex,gt_map_parser_attr* const map_parser_attr,
+    gt_buffered_input_file* const buffered_map_input_master,gt_buffered_input_file* const buffered_map_input_slave); // Used to merge files in parallel
 
 #endif /* GT_INPUT_MAP_PARSER_H_ */
