@@ -2,6 +2,8 @@
  * PROJECT: GEM-Tools library
  * FILE: gt_input_parser.h
  * DATE: 01/06/2012
+ * AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
+ *            Thasso Griebel <thasso.griebel@gmail.com>
  * DESCRIPTION: // TODO
  */
 
@@ -9,7 +11,21 @@
 #define GT_INPUT_PARSER_H_
 
 #include "gt_commons.h"
+#include "gt_data_attributes.h"
 
+#define GT_TAG_PAIR "pair"
+#define GT_TAG_CASAVA "casava"
+#define GT_TAG_EXTRA "extra"
+
+#define GT_PAIR_SE 0
+#define GT_PAIR_BOTH 3
+#define GT_PAIR_PE_1 1
+#define GT_PAIR_PE_2 2
+
+/*
+ * Generic tag parser
+ */
+GT_INLINE gt_status gt_input_parse_tag(char** const text_line,gt_string* const tag,gt_shash* const attributes);
 
 /*
  * Internal Building Blocks for parsing
@@ -41,6 +57,4 @@
     ++(*text_line); \
   }
 
-
-
 #endif /* GT_INPUT_PARSER_H_ */
