@@ -2,12 +2,13 @@
  * PROJECT: GEM-Tools library
  * FILE: gt_buffered_output_file.c
  * DATE: 01/06/2012
+ * AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
  * DESCRIPTION: // TODO
  */
 
 #include "gt_buffered_output_file.h"
 
-#define GT_BUFFERED_OUTPUT_FILE_FORCE_DUMP_SIZE GT_BUFFER_SIZE_32M
+#define GT_BUFFERED_OUTPUT_FILE_FORCE_DUMP_SIZE GT_BUFFER_SIZE_64M
 
 /*
  * Setup
@@ -27,6 +28,7 @@ void gt_buffered_output_file_close(gt_buffered_output_file* const buffered_outpu
     gt_buffered_output_file_dump(buffered_output_file);
     gt_output_file_release_buffer(buffered_output_file->output_file,buffered_output_file->buffer);
   }
+  free(buffered_output_file);
 }
 
 /*
