@@ -2,6 +2,7 @@
  * PROJECT: GEM-Tools library
  * FILE: gt_template_utils.h
  * DATE: 19/07/2012
+ * AUTHOR(S): Santiago Marco-Sola <santiagomsola@gmail.com>
  * DESCRIPTION: // TODO
  */
 
@@ -10,7 +11,7 @@
 /*
  * Template useful functions
  */
-GT_INLINE gt_status gt_template_deduce_alignments_tags(gt_template* const template) {
+GT_INLINE void gt_template_deduce_alignments_tags(gt_template* const template) {
   /*
    * Tag Block Conventions::
    *  (1) line1: NAME => line1&2: NAME
@@ -30,7 +31,49 @@ GT_INLINE gt_status gt_template_deduce_alignments_tags(gt_template* const templa
     register gt_alignment* const alignment = gt_template_get_block(template,i);
     gt_sprintf(alignment->tag,"%s/%"PRIu64,gt_string_get_string(template->tag),i+1);
   }
-  return 0;
+}
+GT_INLINE void gt_template_deduce_template_tag(gt_template* const template,gt_alignment* const alignment) {
+  register uint64_t tag_length = gt_alignment_get_tag_length(alignment);
+  register gt_string* const tag = alignment->tag;
+  if (tag_length>2 && *gt_string_char_at(tag,tag_length-2)==SLASH) {
+    tag_length-=2;
+  }
+  if (tag_length>0) {
+    gt_string_set_nstring(template->tag,gt_string_get_string(tag),tag_length);
+  }
+}
+GT_INLINE void gt_template_dup_tags_to_alignments(gt_template* const template) {
+  register const uint64_t num_blocks = gt_template_get_num_blocks(template);
+  register uint64_t i;
+  for (i=0;i<num_blocks;++i) {
+    register gt_alignment* const alignment = gt_template_get_block(template,i);
+    gt_string_set_nstring(alignment->tag,gt_string_get_string(template->tag),gt_string_get_length(template->tag));
+  }
+}
+GT_INLINE void gt_template_setup_pair_attributes_to_alignments(gt_template* const template,const bool copy_tags) {
+  GT_TEMPLATE_CHECK(template);
+  register const uint64_t num_blocks = gt_template_get_num_blocks(template);
+  int64_t i, p;
+  // Setup all alignments' tags + attr
+  for (i=0;i<num_blocks;i++) {
+    register gt_alignment* const alignment = gt_template_get_block(template,i);
+    if (copy_tags) gt_string_copy(alignment->tag,template->tag);
+    p = (num_blocks>1) ? i+1 : *((int64_t*)gt_attribute_get(template->attributes,GT_TAG_PAIR));
+    gt_attribute_set(alignment->attributes,GT_TAG_PAIR,&p,int64_t);
+    if (gt_shash_is_contained(template->attributes,GT_TAG_CASAVA)) {
+      gt_string* src = gt_shash_get(template->attributes,GT_TAG_CASAVA,gt_string);
+      gt_attribute_set_string(alignment->attributes,GT_TAG_CASAVA,gt_string_dup(src));
+    }
+    if (gt_shash_is_contained(template->attributes, GT_TAG_EXTRA)) {
+      gt_string* src = gt_shash_get(template->attributes,GT_TAG_EXTRA,gt_string);
+      gt_attribute_set_string(alignment->attributes,GT_TAG_EXTRA,gt_string_dup(src));
+    }
+  }
+  // Clear template's pair info
+  if (num_blocks > 1) {
+    p = 0;
+    gt_attribute_set(template->attributes,GT_TAG_PAIR,&p,int64_t);
+  }
 }
 
 /*
@@ -66,6 +109,7 @@ GT_INLINE gt_map** gt_template_raw_put_mmap(
   free(uniq_mmaps); // Free auxiliary vector
   return template_mmap;
 }
+#include "gt_output_map.h"
 GT_INLINE gt_map** gt_template_put_mmap(
     int64_t (*gt_mmap_cmp_fx)(gt_map**,gt_map**,uint64_t),int64_t (*gt_map_cmp_fx)(gt_map*,gt_map*),
     gt_template* const template,gt_map** const mmap,gt_mmap_attributes* const mmap_attr,const bool replace_duplicated) {
@@ -75,23 +119,24 @@ GT_INLINE gt_map** gt_template_put_mmap(
   GT_NULL_CHECK(mmap_attr);
   // Check mmap duplicates
   gt_map** found_mmap;
-  gt_mmap_attributes found_mmap_attr;
-  uint64_t found_mmap_pos;
+  gt_mmap_attributes found_mmap_attr={0,0};
+  uint64_t found_mmap_pos=0;
   register bool is_duplicated = gt_expect_false(gt_template_find_mmap_fx(gt_mmap_cmp_fx,
       template,mmap,&found_mmap_pos,&found_mmap,&found_mmap_attr));
   register gt_map** template_mmap;
-  if (!is_duplicated || replace_duplicated) {
+  if (!is_duplicated || replace_duplicated) { // FIXME: Chose which to replace (like alignment)
     // Resolve mmap aliasing/insertion
     register gt_map** uniq_mmaps = malloc(gt_template_get_num_blocks(template)*sizeof(gt_map*));
     gt_template_alias_dup_mmap_members(gt_map_cmp_fx,template,mmap,uniq_mmaps);
     // Insert mmap
     if (!is_duplicated) { // Add new mmap
-      gt_template_inc_counter(template,mmap_attr->distance+1);
+      gt_template_inc_counter(template,mmap_attr->distance);
       gt_template_add_mmap(template,uniq_mmaps,mmap_attr);
       template_mmap=gt_template_get_mmap(template,gt_template_get_num_mmaps(template)-1,NULL);
     } else { // Replace mmap
-      gt_template_dec_counter(template,found_mmap_attr.distance+1); // Remove old mmap
+      gt_template_dec_counter(template,found_mmap_attr.distance); // Remove old mmap
       gt_template_set_mmap(template,found_mmap_pos,uniq_mmaps,mmap_attr); // Replace old mmap
+      gt_template_inc_counter(template,mmap_attr->distance);
       template_mmap=gt_template_get_mmap(template,found_mmap_pos,NULL);
     }
     free(uniq_mmaps); // Free auxiliary vector
@@ -116,7 +161,7 @@ GT_INLINE void gt_template_insert_mmap_fx(
     gt_template* const template,gt_map** const mmap,gt_mmap_attributes* const mmap_attr) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template);
   GT_NULL_CHECK(mmap); GT_NULL_CHECK(mmap_attr);
-  gt_template_put_mmap(gt_mmap_cmp_fx,gt_map_cmp,template,mmap,mmap_attr,true);
+  gt_template_put_mmap(gt_mmap_cmp_fx,gt_map_cmp,template,mmap,mmap_attr,true); // TODO: Why replace, why?
 }
 GT_INLINE void gt_template_insert_mmap_gtvector(
     gt_template* const template,gt_vector* const mmap,gt_mmap_attributes* const mmap_attr) {
@@ -185,64 +230,81 @@ GT_INLINE bool gt_template_is_mmap_contained_fx(
       &found_mmap_pos,&found_mmap,NULL);
 }
 
+/* Insert size should be and estimate of the original fragment size that was sequenced.  We can get this from the following:
+ *
+ * position of rightmost read + no. bases in rightmost read - position of leftmost read
+ *
+ * If the read has been trimmed from the start of the read then we can't get the original size, but this is a relatively
+ * rare event.  Trimming from the end of the read does not effect the calculation as the rightmost read will be on the
+ * negative strand so trimming x bases from the end of the read will shift the mapped position of the read by x.
+ *
+ * Split mappings require special handling in that we need to consider the number of bases read + the distance between the
+ * last block in each mapping as follows:
+ *
+ * Position of last block of rightmost read + no. bases in rightmost read - (position of last block of leftmost read + no. bases)
+ * in all other blocks of leftmost read
+ */
+GT_INLINE int64_t gt_template_get_insert_size(gt_map** const mmap,uint64_t *gt_error) {
+  // Get last block of each map
+  gt_map *block[2]={0,0};
+  uint64_t length[2]={0,0};
+  int64_t x=0;
+  *gt_error=GT_TEMPLATE_INSERT_SIZE_OK;
+  GT_BEGIN_MAP_BLOCKS_ITERATOR(mmap[0],map_it) {
+    block[0]=map_it;
+    length[0]+=gt_map_get_base_length(block[0]);
+  } GT_END_MAP_BLOCKS_ITERATOR;
+  GT_BEGIN_MAP_BLOCKS_ITERATOR(mmap[1],map_it) {
+    block[1]=map_it;
+    length[1]+=gt_map_get_base_length(block[1]);
+  } GT_END_MAP_BLOCKS_ITERATOR;
+  if (gt_string_equals(block[0]->seq_name,block[1]->seq_name)) {
+    if (block[0]->strand!=block[1]->strand) {
+      if (block[0]->strand==FORWARD) x=1+block[1]->position+length[1]-(block[0]->position+length[0]-gt_map_get_base_length(block[0]));
+      else x=1+block[0]->position+length[0]-(block[1]->position+length[1]-gt_map_get_base_length(block[1]));
+    } else {
+      *gt_error=GT_TEMPLATE_INSERT_SIZE_SAME_STRAND;
+    }
+  } else {
+    *gt_error=GT_TEMPLATE_INSERT_SIZE_DIFFERENT_CONTIGS;
+  }
+  /*  if (gt_map_get_position(mmap[1])>gt_map_get_position(mmap[0])) {
+    return ((int64_t)gt_map_get_position(mmap[1])-(int64_t)(gt_map_get_position(mmap[0])+gt_map_get_length(mmap[0])));
+  } else {
+    return ((int64_t)gt_map_get_position(mmap[0])-(int64_t)(gt_map_get_position(mmap[1])+gt_map_get_length(mmap[1])));
+    } */
+  return x;
+}
+
 /*
  * Template's Counters operators
  */
 GT_INLINE bool gt_template_is_mapped(gt_template* const template) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template);
-  return gt_template_is_thresholded_mapped(template,UINT64_MAX);
+  register const bool unique_flag = gt_template_get_not_unique_flag(template);
+  return unique_flag || gt_template_is_thresholded_mapped(template,UINT64_MAX);
 }
 GT_INLINE bool gt_template_is_thresholded_mapped(gt_template* const template,const uint64_t max_allowed_strata) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template);
   GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
     return gt_alignment_is_thresholded_mapped(alignment,max_allowed_strata);
   } GT_TEMPLATE_END_REDUCTION;
+  if (gt_template_get_not_unique_flag(template)) return true;
+  if(gt_template_get_num_counters(template) == 0) return false;
   register gt_vector* vector = gt_template_get_counters_vector(template);
   GT_VECTOR_ITERATE(vector,counter,counter_pos,uint64_t) {
-    if ((counter_pos+1)>=max_allowed_strata) return false;
+    if (counter_pos>max_allowed_strata) return false;
     else if (*counter!=0) return true;
   }
   return false;
 }
-GT_INLINE int64_t gt_template_get_min_matching_strata(gt_template* const template) {
-  GT_TEMPLATE_CONSISTENCY_CHECK(template);
-  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
-    return gt_alignment_get_min_matching_strata(alignment);
-  } GT_TEMPLATE_END_REDUCTION;
-  register gt_vector* vector = gt_template_get_counters_vector(template);
-  GT_VECTOR_ITERATE(vector,counter,counter_pos,uint64_t) {
-    if (*counter!=0) return counter_pos+1;
-  }
-  return GT_NO_STRATA;
-}
-GT_INLINE int64_t gt_template_get_uniq_degree(gt_template* const template) {
-  GT_TEMPLATE_CONSISTENCY_CHECK(template);
-  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
-    return gt_alignment_get_uniq_degree(alignment);
-  } GT_TEMPLATE_END_REDUCTION;
-  register gt_vector* vector = gt_template_get_counters_vector(template);
-  register bool found_uniq_strata = false;
-  register int64_t uniq_degree = 0;
-  GT_VECTOR_ITERATE(vector,counter,counter_pos,uint64_t) {
-    if (*counter==0) {
-      if (found_uniq_strata) ++uniq_degree;
-    } else if (*counter==1) {
-      if (found_uniq_strata) return uniq_degree;
-      found_uniq_strata=true;
-    } else if (*counter>1) {
-      if (found_uniq_strata) return uniq_degree;
-      return GT_NO_STRATA;
-    }
-  }
-  return GT_NO_STRATA;
-}
 GT_INLINE void gt_template_recalculate_counters(gt_template* const template) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template);
   GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
     gt_alignment_recalculate_counters(alignment);
   } GT_TEMPLATE_END_REDUCTION__RETURN;
   // Clear previous counters
-  gt_vector_clean(gt_template_get_counters_vector(template));
+  gt_vector_clear(gt_template_get_counters_vector(template));
   // Recalculate counters
   register const uint64_t num_blocks = gt_template_get_num_blocks(template);
   GT_TEMPLATE__ATTR_ITERATE_(template,mmap,mmap_attr) {
@@ -251,10 +313,34 @@ GT_INLINE void gt_template_recalculate_counters(gt_template* const template) {
       total_distance+=gt_map_get_global_distance(mmap[i]);
     }
     mmap_attr->distance=total_distance;
-    gt_template_inc_counter(template,total_distance+1);
+    gt_template_inc_counter(template,total_distance);
   }
 }
 
+GT_INLINE int64_t gt_template_get_min_matching_strata(gt_template* const template) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
+    return gt_alignment_get_min_matching_strata(alignment);
+  } GT_TEMPLATE_END_REDUCTION;
+  return gt_counters_get_min_matching_strata(gt_template_get_counters_vector(template));
+}
+GT_INLINE int64_t gt_template_get_uniq_degree(gt_template* const template) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
+    return gt_alignment_get_uniq_degree(alignment);
+  } GT_TEMPLATE_END_REDUCTION;
+  return gt_counters_get_uniq_degree(gt_template_get_counters_vector(template));
+}
+GT_INLINE bool gt_template_get_next_matching_strata(
+    gt_template* const template,const uint64_t begin_strata,
+    uint64_t* const next_matching_strata,uint64_t* const num_maps) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template,alignment) {
+    return gt_alignment_get_next_matching_strata(alignment,begin_strata,next_matching_strata,num_maps);
+  } GT_TEMPLATE_END_REDUCTION;
+  return gt_counters_get_next_matching_strata(gt_template_get_counters_vector(template),
+      begin_strata,next_matching_strata,num_maps);
+}
 
 /*
  * Template Set operators
@@ -262,6 +348,13 @@ GT_INLINE void gt_template_recalculate_counters(gt_template* const template) {
 GT_INLINE void gt_template_merge_template_mmaps(gt_template* const template_dst,gt_template* const template_src) {
   GT_TEMPLATE_CONSISTENCY_CHECK(template_dst);
   GT_TEMPLATE_CONSISTENCY_CHECK(template_src);
+  GT_TEMPLATE_COMMON_CONSISTENCY_ERROR(template_dst,template_src);
+  GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template_src,alignment_src) {
+    GT_TEMPLATE_IF_REDUCES_TO_ALINGMENT(template_dst,alignment_dst) {
+      gt_alignment_merge_alignment_maps(alignment_dst,alignment_src);
+    } GT_TEMPLATE_END_REDUCTION;
+  } GT_TEMPLATE_END_REDUCTION__RETURN;
+  // Merge mmaps
   gt_template_merge_template_mmaps_fx(gt_mmap_cmp,gt_map_cmp,template_dst,template_src);
 }
 GT_INLINE void gt_template_merge_template_mmaps_fx(
@@ -278,10 +371,11 @@ GT_INLINE void gt_template_merge_template_mmaps_fx(
   GT_TEMPLATE_CONSISTENCY_CHECK(template_dst);
   GT_TEMPLATE_CONSISTENCY_CHECK(template_src);
   GT_TEMPLATE__ATTR_ITERATE(template_src,mmap,mmap_attr) {
-    register gt_map** mmap_copy = gt_mmap_array_copy(mmap,__map_array_num_blocks);
-    gt_template_put_mmap(gt_mmap_cmp_fx,gt_map_cmp_fx,template_dst,mmap_copy,mmap_attr,false);
+    register gt_map** mmap_copy = gt_mmap_array_copy(mmap,__mmap_num_blocks);
+    gt_template_put_mmap(gt_mmap_cmp_fx,gt_map_cmp_fx,template_dst,mmap_copy,mmap_attr,true);
     free(mmap_copy); // Free array handler
   }
+  gt_template_set_mcs(template_dst,GT_MIN(gt_template_get_mcs(template_dst),gt_template_get_mcs(template_src)));
 }
 
 //// TODO: Scheduled for v2.0
@@ -304,17 +398,62 @@ GT_INLINE void gt_template_merge_template_mmaps_fx(
 //  }
 //}
 
+GT_INLINE gt_template* gt_template_union_template_mmaps_v(
+    const uint64_t num_src_templates,gt_template* const template_src,va_list v_args) {
+  GT_ZERO_CHECK(num_src_templates);
+  // Create new template
+  register gt_template* const template_union = gt_template_copy(template_src,false,false);
+  gt_template_merge_template_mmaps(template_union,template_src);
+  // Merge template sources into template_union
+  register uint64_t num_tmp_merged = 1;
+  while (num_tmp_merged < num_src_templates) {
+    register gt_template* template_target = va_arg(v_args,gt_template*);
+    GT_TEMPLATE_COMMON_CONSISTENCY_ERROR(template_union,template_target);
+    GT_TEMPLATE_CONSISTENCY_CHECK(template_target);
+    gt_template_merge_template_mmaps(template_union,template_target);
+    ++num_tmp_merged;
+  }
+  return template_union;
+}
+GT_INLINE gt_template* gt_template_union_template_mmaps_va(
+    const uint64_t num_src_templates,gt_template* const template_src,...) {
+  GT_ZERO_CHECK(num_src_templates);
+  GT_TEMPLATE_CONSISTENCY_CHECK(template_src);
+  va_list v_args;
+  va_start(v_args,template_src);
+  register gt_template* const template_union =
+      gt_template_union_template_mmaps_v(num_src_templates,template_src,v_args);
+  va_end(v_args);
+  return template_union;
+}
+GT_INLINE gt_template* gt_template_union_template_mmaps_a(
+    gt_template** const templates,const uint64_t num_src_templates) {
+  GT_ZERO_CHECK(num_src_templates);
+  // Create new template
+  register gt_template* const template_union = gt_template_copy(templates[0],false,false);
+  gt_template_merge_template_mmaps(template_union,templates[0]);
+  // Merge template sources into template_union
+  register uint64_t i;
+  for (i=1;i<num_src_templates;++i) {
+    GT_TEMPLATE_COMMON_CONSISTENCY_ERROR(template_union,templates[i]);
+    GT_TEMPLATE_CONSISTENCY_CHECK(templates[i]);
+    gt_template_merge_template_mmaps(template_union,templates[i]);
+  }
+  return template_union;
+}
+
 GT_INLINE gt_template* gt_template_union_template_mmaps_fx_v(
     int64_t (*gt_mmap_cmp_fx)(gt_map**,gt_map**,uint64_t),int64_t (*gt_map_cmp_fx)(gt_map*,gt_map*),
     const uint64_t num_src_templates,gt_template* const template_src,va_list v_args) {
   GT_NULL_CHECK(gt_mmap_cmp_fx);
   GT_ZERO_CHECK(num_src_templates);
   // Create new template
-  register gt_template* const template_union = gt_template_copy(template_src,true,true);
+  register gt_template* const template_union = gt_template_copy(template_src,false,false);
+  gt_template_merge_template_mmaps_fx(gt_mmap_cmp_fx,gt_map_cmp_fx,template_union,template_src);
   // Merge template sources into template_union
-  register uint64_t num_tmp_merged = 0;
+  register uint64_t num_tmp_merged = 1;
   while (num_tmp_merged < num_src_templates) {
-    register gt_template* template_target = (gt_expect_true(num_tmp_merged>0)) ? va_arg(v_args,gt_template*) : template_src;
+    register gt_template* template_target = va_arg(v_args,gt_template*);
     GT_TEMPLATE_COMMON_CONSISTENCY_ERROR(template_union,template_target);
     GT_TEMPLATE_CONSISTENCY_CHECK(template_target);
     gt_template_merge_template_mmaps_fx(gt_mmap_cmp_fx,gt_map_cmp_fx,template_union,template_target);
@@ -335,17 +474,6 @@ GT_INLINE gt_template* gt_template_union_template_mmaps_fx_va(
   va_end(v_args);
   return template_union;
 }
-GT_INLINE gt_template* gt_template_union_template_mmaps_va(
-    const uint64_t num_src_templates,gt_template* const template_src,...) {
-  GT_ZERO_CHECK(num_src_templates);
-  GT_TEMPLATE_CONSISTENCY_CHECK(template_src);
-  va_list v_args;
-  va_start(v_args,template_src);
-  register gt_template* const template_union =
-      gt_template_union_template_mmaps_fx_v(gt_mmap_cmp,gt_map_cmp,num_src_templates,template_src,v_args);
-  va_end(v_args);
-  return template_union;
-}
 
 GT_INLINE gt_template* gt_template_subtract_template_mmaps_fx(
     int64_t (*gt_mmap_cmp_fx)(gt_map**,gt_map**,uint64_t),int64_t (*gt_map_cmp_fx)(gt_map*,gt_map*),
@@ -371,7 +499,7 @@ GT_INLINE gt_template* gt_template_subtract_template_mmaps_fx(
   gt_mmap_attributes found_mmap_attr;
   GT_TEMPLATE__ATTR_ITERATE(template_minuend,mmap,mmap_attr) {
     if (!gt_template_find_mmap_fx(gt_mmap_cmp_fx,template_subtrahend,mmap,&found_mmap_pos,&found_mmap,&found_mmap_attr)) {
-      register gt_map** mmap_copy = gt_mmap_array_copy(mmap,__map_array_num_blocks);
+      register gt_map** mmap_copy = gt_mmap_array_copy(mmap,__mmap_num_blocks);
       gt_template_put_mmap(gt_mmap_cmp_fx,gt_map_cmp_fx,template_difference,mmap_copy,mmap_attr,false);
       free(mmap_copy);
     }
@@ -409,7 +537,7 @@ GT_INLINE gt_template* gt_template_intersect_template_mmaps_fx(
   gt_mmap_attributes found_mmap_attr;
   GT_TEMPLATE__ATTR_ITERATE(template_A,mmap,mmap_attr) {
     if (gt_template_find_mmap_fx(gt_mmap_cmp_fx,template_B,mmap,&found_mmap_pos,&found_mmap,&found_mmap_attr)) {
-      register gt_map** mmap_copy = gt_mmap_array_copy(mmap,__map_array_num_blocks);
+      register gt_map** mmap_copy = gt_mmap_array_copy(mmap,__mmap_num_blocks);
       gt_template_put_mmap(gt_mmap_cmp_fx,gt_map_cmp_fx,template_intersection,mmap_copy,mmap_attr,false);
       free(mmap_copy);
     }
@@ -422,3 +550,65 @@ GT_INLINE gt_template* gt_template_intersect_template_mmaps(
   GT_TEMPLATE_CONSISTENCY_CHECK(template_B);
   return gt_template_intersect_template_mmaps_fx(gt_mmap_cmp,gt_map_cmp,template_A,template_B);
 }
+
+/*
+ * Template realignment
+ */
+GT_INLINE void gt_template_recover_mismatches(gt_template* const template,gt_sequence_archive* const sequence_archive) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_SEQUENCE_ARCHIVE_CHECK(sequence_archive);
+  GT_TEMPLATE_ALIGNMENT_ITERATE(template,alignment) {
+    gt_alignment_recover_mismatches(alignment,sequence_archive);
+  }
+  if (gt_template_get_num_blocks(template)>1) gt_template_recalculate_counters(template);
+}
+GT_INLINE void gt_template_realign_hamming(gt_template* const template,gt_sequence_archive* const sequence_archive) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_SEQUENCE_ARCHIVE_CHECK(sequence_archive);
+  GT_TEMPLATE_ALIGNMENT_ITERATE(template,alignment) {
+    gt_alignment_realign_hamming(alignment,sequence_archive);
+  }
+  if (gt_template_get_num_blocks(template)>1) gt_template_recalculate_counters(template);
+}
+GT_INLINE void gt_template_realign_levenshtein(gt_template* const template,gt_sequence_archive* const sequence_archive) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_SEQUENCE_ARCHIVE_CHECK(sequence_archive);
+  GT_TEMPLATE_ALIGNMENT_ITERATE(template,alignment) {
+    gt_alignment_realign_levenshtein(alignment,sequence_archive);
+  }
+  if (gt_template_get_num_blocks(template)>1) gt_template_recalculate_counters(template);
+}
+GT_INLINE void gt_template_realign_weighted(
+    gt_template* const template,gt_sequence_archive* const sequence_archive,int32_t (*gt_weigh_fx)(char*,char*)) {
+  GT_TEMPLATE_CONSISTENCY_CHECK(template);
+  GT_SEQUENCE_ARCHIVE_CHECK(sequence_archive);
+  GT_NULL_CHECK(gt_weigh_fx);
+  GT_TEMPLATE_ALIGNMENT_ITERATE(template,alignment) {
+    gt_alignment_realign_weighted(alignment,sequence_archive,gt_weigh_fx);
+  }
+  if (gt_template_get_num_blocks(template)>1) gt_template_recalculate_counters(template);
+}
+
+/*
+ * Template trimming
+ */
+GT_INLINE void gt_template_trim(gt_template* const template, uint64_t const left, uint64_t const right, uint64_t const min_length, bool const set_extra){
+  GT_TEMPLATE_ALIGNMENT_ITERATE(template,alignment) {
+    gt_alignment_trim(alignment, left, right, min_length, set_extra);
+  }
+  if(set_extra){
+    gt_alignment* first = gt_template_get_block(template, 0);
+    // transfer extra attribute
+    gt_string* alignment_extra = gt_shash_get(first->attributes, GT_TAG_EXTRA, gt_string);
+
+    if(gt_shash_is_contained(template->attributes, GT_TAG_EXTRA)){
+      gt_string* old = gt_shash_get(template->attributes, GT_TAG_EXTRA, gt_string);
+      gt_string_copy(old, alignment_extra);
+    }else{
+      gt_string* extra = gt_string_new(8+ (2*left) + (2*right));
+      gt_string_copy(extra, alignment_extra);
+      gt_shash_insert(template->attributes, GT_TAG_EXTRA, extra, gt_string);
+    }
+  }
+}
+
